#enum-panel .panel{width:80%;max-width:1080px;}
#enum-list{padding-bottom:100px;}
#enum-list .is-empty{font-size:20px;text-align:center;color:#aaaaaa;}
#enum-list .enum-table{margin-bottom:20px;}
#enum-list .enum-table .enum--header-index,#enum-list .enum-table .enum--header-value,#enum-list .enum-table .enum--item-index,#enum-list .enum-table .enum--item-value{text-align:center;}
#enum-list .enum-table .enum--item-index{font-size:12px;}
#enum-list .enum-table th:not(.enum--header-index){min-width:60px;}
#enum-list .enum-table td :not(:first-child),#enum-list .enum-table th :not(:first-child){padding:5px 10px;}
#enum-list .enum-index{display:inline-block;margin-right:10px;font-size:12px;}
#enum-list .enum-name{font-weight:bold;margin:10px 0;transition-property:font-size,color;transition-duration:500ms;transition-timing-function:ease-in-out;transition-delay:200ms;font-family:Consolas,Monospaced,FangSong,serif;font-size:20px;}
#enum-list .enum-title.highlight .enum-name{font-size:80px;color:#00a6ff;}
#enum-list .enum--item-value{text-align:center;}
#enum-list .enum-comment.route-description{height:auto;}