#option-panel .options{font-size:12px;}
#option-panel .options select{width:140px;height:26px;margin-right:20px;border-width:0 0 1px 0;}
#option-panel .option-tip{padding:10px;color:#555555;font-size:12px;text-align:right;}
#option-panel .option-tip:before{content:'!';width:16px;height:16px;line-height:16px;background:#888888;color:#ffffff;display:inline-block;border-radius:50%;text-align:center;overflow:hidden;font-family:'黑体',serif;vertical-align:middle;}
#option-panel .panel{width:800px;height:480px;}
#option-panel .options{padding:10px 60px;}
#option-panel .options>div{margin:10px 0;}
#option-panel input[type='checkbox']{vertical-align:-2px;}
.theme-dark #option-panel .option-tip{color:#c4c3c3;}