.tabs-layout{width:100%;margin-bottom:40px;}
.tabs-item{width:100%;display:none;}
.tabs-item.tabs-layout>.tabs-header{border:none;}
.tabs-item.tabs-layout>.tabs-header>li.active{border:none;color:#0a69bd;border-bottom:1px solid #0a69bd;}
.tabs-item.tabs-layout>.tabs-header>li:hover{border:none;border-bottom:1px solid #0a69bd;}
div.tabs-item.active{display:block;}
table.tabs-item.active{display:table;}
.tabs-header{list-style:none;padding:0;border-bottom:1px solid #cccccc;overflow:auto;margin:10px 0 5px;display:flex;flex-direction:row;}
.tabs-header>li[data-index]{padding:3px 10px;margin-left:5px;cursor:pointer;color:#666666;flex:0 0 auto;}
.tabs-header>li[data-index]:hover,.tabs-header>li[data-index].active{border:1px solid #cccccc;border-bottom:none;color:#333333;}
.tabs-header>li:not([data-index]){flex:1 1 auto;padding-left:20px;}
.theme-dark .tabs-header{border-bottom-color:#858585;}
.theme-dark .tabs-header>li[data-index]{color:#aaaaaa;}
.theme-dark .tabs-header>li[data-index]:hover,.theme-dark .tabs-header>li[data-index].active{border-color:#858585;color:#dddddd;}