.panel-container{position:fixed;left:0;top:0;right:0;bottom:0;background-color:rgba(0,0,0,0.2);align-items:center;justify-content:center;display:none;transition:background-color 618ms;z-index:2;}
.panel{background-color:#ffffff;border-radius:4px;transition:width 200ms,height 200ms;display:flex;flex-direction:column;height:100%;}
.panel.fullscreen{max-width:unset !important;width:100% !important;height:100% !important;}
.panel-heading{height:45px;padding:10px;box-sizing:border-box;border-bottom:1px solid #dddddd;user-select:none;font-size:1.1em;flex:0 0 auto;display:flex;justify-content:space-between;}
.panel-heading--buttons button{border:none;}
.panel-body{padding:5px 10px;overflow:auto;flex:1 1 auto;}
.theme-dark .panel-container{background-color:rgba(29,29,29,0.5);}
.theme-dark .panel{background-color:#555555;}
.theme-dark .panel .panel-heading{border-bottom-color:#474747;}