#api-list{min-height:200px;padding:0 10px;box-sizing:border-box;}
details{clear:both;}
details summary{cursor:default;outline:none;font-size:large;color:#444444;display:block;padding:10px;background-color:#ffffff;transition:background-color 618ms;}
.module-name:before,.route-name:before{padding:2px 5px;border-radius:2px;margin:0 5px;vertical-align:1px;background:#ececec;color:#636363;}
.module-name:before{content:'模块';font-size:13px;}
.route-name:before{content:'路由';font-size:12px;}
.route-description,.return-info{background-color:#fafafa;padding:10px;border-radius:4px;line-height:1.6;}
.route-count{margin-left:20px;font-size:12px;line-height:1;color:#0977c0;}
#api-list[data-select-type='1'] .url-info .url,#api-list[data-select-type='2'] .url-info .url-prefix-with-slash,#api-list[data-select-type='3'] .url-info .url-prefix,#api-list[data-select-type='4'] .url-info .url-path-with-slash,#api-list[data-select-type='5'] .url-info .url-path{user-select:all;border-bottom:1px dotted;}
.method{text-transform:uppercase;color:#e90588;padding:0 5px;}
.method-get .method{color:#16930a;}
.method-get .url{color:#16930a;}
.method-post .method{color:#0989f7;}
.method-post .url{color:#0989f7;}
.method-put .method{color:#06b0c1;}
.method-put .url{color:#06b0c1;}
.method-delete .method{color:#f9512b;}
.method-delete .url{color:#f9512b;}
.args-table{width:100%;border-collapse:collapse;font-size:14px;}
.args-table caption{text-align:left;padding-bottom:5px;}
.args-table th{font-weight:normal;}
.arg-type.is-variable{color:#d462f2;}
.arg-type.is-enum{border-bottom:1px dotted;cursor:pointer;}
.arg-type.is-enum:hover,.arg-type.is-enum:active{border-bottom-style:solid;}
.required-field{color:#d30505;}
.arg-name{padding:2px 3px;}
.api-items li.module-item{list-style:simp-chinese-informal;}
@keyframes anim-hide{0%{opacity:1;transform:scaleY(1);}
100%{transform:scaleY(0);opacity:0;height:0;}}
@keyframes anim-show{0%{transform:scaleY(0);height:0;opacity:0;}
100%{transform:scaleY(1);opacity:1;height:auto;}}
.route-description,.addition-info,.args-table,.return-info{animation-fill-mode:forwards;animation-duration:618ms;animation-timing-function:ease-in-out;height:0;overflow:hidden;}
.api-item.collapsed{margin-bottom:0;}
.api-item.collapsed .addition-info,.api-item.collapsed .args-table,.api-item.collapsed .return-info,.api-item.collapsed .route-description{display:block;animation-name:anim-hide;overflow:hidden;margin:0;padding:0;}
.api-item.expanded .addition-info,.api-item.expanded .args-table,.api-item.expanded .return-info,.api-item.expanded .route-description{animation-name:anim-show;}
.api-item-title{cursor:default;display:flex;align-items:center;}
.api-item-title:hover{background-color:rgba(0,0,0,.2);}
.api-item-title .anchor{width:20px;text-align:center;flex:0 0 auto;}
.api-item-title:hover .api-item-arrow:before,.api-item-title:hover .api-item-arrow:after{border-color:transparent #0787fd #0787fd transparent;}
.route-name{flex:1 1 auto;}
.api-item-arrow{display:inline-block;width:30px;height:30px;flex:0 0 auto;position:relative;cursor:pointer;transition:transform 312ms;}
.api-item-arrow:before,.api-item-arrow:after{content:'';position:absolute;display:inline-block;border-color:transparent rgba(142,194,246,0.74)rgba(142,194,246,0.74)transparent;border-width:1px;border-style:solid;width:10px;height:10px;transform:rotate(45deg);transition:border-color 618ms;}
.api-item-arrow:before{top:9px;left:8px;}
.api-item-arrow:after{top:3px;left:8px;}
.collapsed .api-item-arrow{transform:rotate(-90deg);}
details summary:hover{background-color:rgba(0,0,0,.2);}
details summary:hover .anchor,details ol li:hover .anchor{opacity:1;}
details ol li.api-item{border-left:1px dotted transparent;list-style:decimal-leading-zero;padding-left:5px;padding-top:10px;margin-bottom:40px;}
details ol li.api-item:hover{border-left-color:#539fdd;}
details ol li.api-item:hover .btn-open-test{opacity:1;}
details[open]{margin-bottom:20px;}
details[open] summary{position:sticky;top:-1px;border-bottom:1px solid #eeeeee;z-index:1;}
.url-info{display:flex;flex-direction:row;margin:15px 0;align-items:center;}
.info{padding:5px 0;border-radius:2px;transition:background-color 200ms;will-change:background-color;flex:1 1 auto;}
.info:hover{background-color:#fdf4ff;}
.btn-open-test{width:60px;flex:0 0 auto;}
.addition-info{margin:15px 5px;}
.comment{font-size:12px;color:#555555;}
.comment pre{line-height:1.4;margin:5px 0;padding:0 5px;}
.route-name{font-size:15px;margin-right:10px;}
.return-info{font-size:1em;line-height:1.6;}
.return-info code{margin-right:5px;}
#current-anchor{font-weight:bold;box-shadow:0 0 10px 2px rgba(180,180,180,0.58);}
.theme-dark details summary{background-color:#555555;color:#eeeeee;}
.theme-dark details summary:hover{background-color:#626262;}
.theme-dark details[open] summary{border-bottom-color:#4a4a4a;}
.theme-dark .anchor{color:#3f86cc;}
.theme-dark .module-name:before,.theme-dark .route-name:before{color:#ececec;background-color:#717070;}
.theme-dark .route-description,.theme-dark .return-info{background-color:#505050;}
.theme-dark .info:hover{background-color:#444145;}
.theme-dark .required-field{color:#ff7d7d;}
.theme-dark .arg-type.is-variable{color:#ff57f4;}
.theme-dark .route-count{color:#6bbbf0;}