#api-nav{position:fixed;left:0;top:80px;bottom:60px;transition:width 100ms;}
#api-nav ul{margin:0;list-style:none;padding:0;}
#api-nav .header{padding:5px 10px;font-size:large;font-weight:bold;box-sizing:border-box;user-select:none;border-left:3px solid;}
#api-nav .header small{font-size:smaller;color:#1b78ce;cursor:pointer;}
#api-nav .header small:hover{color:red;}
#app.nav-collapsed #api-nav{width:auto;}
#app.nav-collapsed #api-nav #api-nav-list{display:none;}
#app.nav-collapsed #api-nav .header{padding:0;display:flex;justify-content:center;align-items:center;width:24px;border:none;}
#app.nav-collapsed #api-nav .header span{cursor:pointer;border-width:1px 1px 1px 0;border-style:solid;padding:5px;font-weight:normal;font-size:small;border-radius:0 4px 4px 0;}
#app.nav-collapsed #api-nav .header small{display:none;}
#api-nav-list{position:relative;margin-left:5px !important;height:100%;width:100%;overflow:auto;}
#api-nav-list ul{margin-left:15px;padding-left:5px;border-left:1px dotted;}
#api-nav-list li{padding:0;margin:0;}
#api-nav-list .module-name{font-size:larger;padding:5px 0;}
#api-nav-list .route-name{padding:2px 0;word-break:keep-all;white-space:nowrap;}
#api-nav-list a{text-decoration:none;}
@media screen and (max-width:1400px){#app:not(.nav-collapsed) #api-nav{position:relative;display:block;width:auto;top:unset;bottom:unset;margin-left:40px;}
#app:not(.nav-collapsed) .scroll-proxy{margin-left:auto;}}
@media screen and (min-width:1400px){#app:not(.nav-collapsed) #api-nav{display:block;width:260px;}
#app:not(.nav-collapsed) .scroll-proxy{margin-left:20%;}}
@media screen and (min-width:1600px){#app:not(.nav-collapsed) #api-nav{display:block;width:380px;}
#app:not(.nav-collapsed) .scroll-proxy{margin-left:23%;}}
@media screen and (min-width:1900px){#app:not(.nav-collapsed) #api-nav{display:block;width:440px;}
#app:not(.nav-collapsed) .scroll-proxy{margin-left:27%;}}