#test-panel .panel{min-width:720px;max-width:1080px;width:80%;height:auto;max-height:100%;}
#test-panel .url-info .info{display:flex;justify-content:center;align-items:center;}
#test-panel .url-info .info .method{flex:0 0 auto;}
#test-panel .url-info .info .url{border:1px solid #c4c3c3;outline:none;flex:1 1 auto;height:24px;}
#test-panel .url-info .info .url:focus,#test-panel .url-info .info .url:active{border-color:#888888;}
#test-panel .test-row--tool td{padding:5px;}
#test-panel .test-lead{background-color:#bebdbd;color:#ffffff;border-radius:2px;padding:2px 3px;margin-right:5px;font-size:12px;}
#test-panel input.arg-name:focus,#test-panel .arg-value:focus,#test-panel input.arg-name:hover,#test-panel .arg-value:hover,#test-panel input.arg-name:active,#test-panel .arg-value:active{border-width:1px;box-shadow:0 0 3px 2px #bab9b9;padding:5px;}
#test-panel input[type=file].arg-value:focus,#test-panel input[type=file].arg-value:hover,#test-panel input[type=file].arg-value:active{padding:2px;}
#test-panel select.arg-type{height:26px;}
#test-panel .btn-open-test{margin-left:20px;font-size:13px;opacity:0.2;will-change:opacity;transition:opacity 200ms;}
#test-panel #btn-send-test{font-size:16px;letter-spacing:5px;height:30px;}
#test-panel .response p{font-size:14px;}
#test-panel .response-image{max-width:100%;}
#test-panel .response-image:hover{box-shadow:0 0 10px 2px #b7b7b7;background-color:#ffffff;}
#test-panel .response-content{background:#ececeb;border:none;padding:15px;width:100%;box-sizing:border-box;outline:none;font-size:12px;}
#test-panel .response-content,#test-panel .response-content>pre{word-break:break-all;line-height:1.5;white-space:break-spaces;}
#test-panel #test-response--save-as{margin-right:20px;}
#test-panel .response-status{margin-right:20px;}
#test-panel .status-success{color:green;}
#test-panel .status-failure{color:orange;}
#test-panel .response-time,#test-panel .response-type,#test-panel .response-length{margin-right:20px;font-size:12px;}
#test-panel tr.test-row--tool{background-color:#ffffff !important;}
#test-panel #btn-add-test-field,#test-panel #btn-add-test-header,#test-panel .btn-remove-test-field,#test-panel .btn-remove-test-header,#test-panel .btn-remove-test-header{font-size:12px;color:#333333;}
#test-panel #btn-add-test-field:hover,#test-panel #btn-add-test-header:hover{color:#1188ed;}
#test-panel .btn-remove-test-field:hover,#test-panel .btn-remove-test-header:hover{color:#eb5a5a;}
#test-panel .response-meta{float:right;text-align:right;}
#test-panel .response-header td{word-break:break-all;white-space:pre-wrap;}
#test-panel .test-header--checkbox{cursor:pointer;}
#test-panel #args-json{height:auto;min-height:200px;border:1px solid #cccccc;border-radius:4px;padding:10px;box-sizing:border-box;resize:vertical;margin-top:10px;font-family:Consolas,serif;line-height:1.2;}
#test-panel .args-json--toolbar{height:40px;}
#test-panel #json-tool--format{float:left;}
#test-panel .args-json--tip{margin-left:60px;overflow:hidden;text-overflow:ellipsis;word-break:break-all;white-space:nowrap;font-size:12px;padding-top:2px;}
#test-panel .args-json--tip.error{color:#fb6060;}
#test-panel .args-json--tip.ok{color:#0b7500;}
.theme-dark #test-panel input.arg-name{color:#00c7ff;}
.theme-dark #test-panel .response-content{background-color:#4c4c4c;}
.theme-dark #test-panel tr.test-row--tool{background-color:#5f5f5f !important;}
.theme-dark #test-panel #btn-add-test-field,.theme-dark #test-panel #btn-add-test-header,.theme-dark #test-panel .btn-remove-test-field,.theme-dark #test-panel .btn-remove-test-header{color:#eeeeee;}
.theme-dark #test-panel #btn-add-test-field:hover,.theme-dark #test-panel #btn-add-test-header:hover,.theme-dark #test-panel .btn-remove-test-field:hover{color:#00c7ff;}
.theme-dark #test-panel .test-lead{color:#ececec;background-color:#717070;}
.theme-dark #test-panel .url{border:1px solid #717070;}
.theme-dark #test-panel input.arg-name:focus,.theme-dark #test-panel .arg-value:focus,.theme-dark #test-panel input.arg-name:hover,.theme-dark #test-panel .arg-value:hover,.theme-dark #test-panel input.arg-name:active,.theme-dark #test-panel .arg-value:active{box-shadow:0 0 3px 2px #717070;}
.theme-dark #test-panel .response-image{background-color:#4a4a4a;}