#search-container{display:inline-block;width:260px;}
#search-container.focused{position:fixed;left:0;top:0;width:100%;z-index:2;display:flex;justify-content:center;align-items:flex-start;}
#search-container.focused #search-input{width:800px;height:80px;font-size:30px;padding:0 20px;box-shadow:0 0 30px 10px #6e6f6f;border-radius:0 0 12px 12px;border-width:0 1px 1px 1px;outline:none;transition:width 618ms;}
#search-input{width:260px;max-width:600px;height:32px;padding:2px 5px;box-sizing:border-box;outline:none;border-radius:4px;border:1px solid #cdcdcd;color:#333333;will-change:width;font-family:'Consolas','Monaco','Bitstream Vera Sans Mono',monospace;}
#search-input:focus{border-color:#3f86cc;}
#search-input::placeholder{color:#bbbbbb;}
.search-miss{display:none;}
.search-hl{background-color:#e033b6;color:#ffffff;padding:1px;margin:0 2px;border-radius:2px;}
.theme-dark #search-input{background:#424040;border-color:#444444;color:#eeeeee;}
.theme-dark #search-input::placeholder{color:#888888;}